

/* "grow3 extends water" means that there are a lot 
	of routines in water.java that this applet imports
	and uses, so if you want to understand this code, 
	read and understand water.java first.
	
	The routines in this file replace those in water.java
	that have the same names. This is called "class inheritance".
	This allows a small amount of code to slightly modify 
	existing programs that can be very large. 
		
	This is not, technically, an l-system, since there's no string
	replacement, but it is inspired by "The Algorithmic Beauty of Plants."
	
	Goal: grow unti you run out of particles. 
	Then use existing positions held by particles to grow
	
	I am Neal McDonald, this is my code, whosoever should
	leave in a pointer to me:
	www.workly.com
	*/

import java.awt.*;
import java.awt.image.*;
import java.applet.*;


public class grow3 extends water {
	
	final static int TYPE=10;
	final static int PARENT=11;		// owner; use to get relative posn.
	final static int STRENGTH=12;
	final static int BIRTHDAY=13;	// set==simTick when part's added
	final static int LENGTH=14;
	final static int DDIR=15;
	final static int DDDIR=16;
	
	/* values   to put int parts[][TYPE] */
	final static int DEAD=-1;
	final static int SEED=1;	// base particle; use seedXY
	final static int STEM1=2;
	final static int PETAL=3;
	final static int STEM2=4;
	final static int STEM3=5;
	final static int STEM4=6;
	
	int p2init; 		// index of the next particle to use	
	int seedX, seedY;	// location of seed particle
	int stems;

	Color	stemColor;

	void initPart(int which) {
		if (which==0) {
			//	System.out.println("addPart: in SEED");
				parts[which][TYPE] = (double)SEED;
				parts[which][XPOS] = (double)seedX;
				parts[which][YPOS] = (double)seedY;
				parts[which][OX] = (double)seedX;
				parts[which][OY] = (double)seedY;
				parts[which][STRENGTH] = 10.0;
				parts[which][LENGTH] = 0.0;
				parts[which][DIR] = 180.0;
				parts[which][DDIR] = 2.0;
				parts[which][DDDIR] = 0.1;
				partColors[which] = Color.yellow;
		} else {
			parts[which][TYPE] = (double)DEAD;
		}	
	}





	/* activate a particle. */
	void addPart(int source, int type) {
		int i, intype;
		double p;
		
		i=1;
		if (p2init==source) { i=0; }
		intype = (int)parts[p2init][TYPE];
		switch (intype) {
			case SEED: i=0; break;
			case STEM1: if (Math.random()<0.4) { i=0; } break;
			case STEM2: if (Math.random()<0.1) { i=0; } break;
			case STEM3: if (Math.random()<0.1) { i=0; } break;
		}		
		
		if (i==1) {
			// remove use of part #p2init as parent
			for (i=1; i<NUM_PARTS; ++i) {
				p = parts[i][PARENT];
				if (p>0.5) { // unless parent is the seed
					p=p-(double)p2init;
					if ((p*p)<0.1) {
						parts[i][PARENT] = -10.0;
					}
				}
			}	

			parts[p2init][TYPE] = (double)type;
			parts[p2init][PARENT] = (double)source;
			parts[p2init][BIRTHDAY] = (double)simTicks;

			switch (type) {
			// seeds are stationary; generate stems
			// they have buds on their ends
			case SEED:
				parts[p2init][XPOS] = (double)seedX;
				parts[p2init][YPOS] = (double)seedY;
				parts[p2init][DIR] = Math.random()*360.0;
				parts[p2init][DDIR] = 160.0;
				parts[p2init][DDDIR] = 0.01;
				parts[p2init][STRENGTH] = 10.0;
				parts[p2init][LENGTH] = 0.0;
				partColors[p2init] = Color.yellow;
				break;
				// stems have a length that increases
			case STEM1:
				parts[p2init][DIR] = parts[source][DIR] +(Math.random()*40.0)-20.0;
				parts[p2init][DDIR] = 0.1;
				parts[p2init][DDDIR] = 0.1;
				parts[p2init][STRENGTH] = Math.random()*.1;
				parts[p2init][LENGTH] = 1.0;
				partColors[p2init] = stemColor;
				break;
			case STEM2:
				parts[p2init][DIR] = parts[source][DIR] +(Math.random()*10.0)-5.0;
				parts[p2init][DDIR] = 0.1;
				parts[p2init][DDDIR] = 0.1;
				parts[p2init][STRENGTH] = Math.random()*.2;
				parts[p2init][LENGTH] = 1.0;
				partColors[p2init] = stemColor;
				break;
			case STEM3:
				parts[p2init][DIR] = parts[source][DIR] +(Math.random()*80.0)-40.0;
				parts[p2init][DDIR] = 0.1;
				parts[p2init][DDDIR] = 0.1;
				parts[p2init][STRENGTH] = Math.random()*.5;
				parts[p2init][LENGTH] = 1.0;
				partColors[p2init] = stemColor;
				break;
				// petals generate marks
			case PETAL:
				parts[p2init][DIR] = Math.random()*360.0;
				parts[p2init][DDIR] = 0.1;
				parts[p2init][DDDIR] = 0.01;
				parts[p2init][STRENGTH] = Math.random()*.2;
				parts[p2init][LENGTH] = 0.0;
				partColors[p2init] = Color.red;
				break;
				}
			++p2init;
			if (p2init==NUM_PARTS) { p2init=1; }	
		}
	}	
	
	
	
	void initMore() {
		seedX= 
			java.lang.Integer.parseInt(getParameter("seedX"));
		seedY= 
			java.lang.Integer.parseInt(getParameter("seedY"));
			
		p2init = 1;
		stemColor = new Color(70,90,65);
	}
	



	void drawPart(Graphics gr, int which) {
		int type, px,py,ox,oy;
		double r, wear;
		
		type = (int)parts[which][TYPE];
		if (Math.random()<0.08) {
			r_setColor(gr, backgroundColor);
		} else {
			r_setColor(gr, partColors[which]);
		}		
		px = (int)parts[which][XPOS];
		py = (int)parts[which][YPOS];
		ox = (int)parts[which][OX];
		oy = (int)parts[which][OY];

		switch (type) {
		case SEED: 
			r_drawLine(gr, px, py, px-4, py);
			r_drawLine(gr, px, py, px, py+3);
			r_drawLine(gr, px, py, px+4, py);
			break;
		case STEM1: 
		case STEM2: 
		case STEM3: 
			partColors[which] = getColorC(px, py);
			r_setColor(gr, partColors[which]);
			r_drawLine(gr, px, py, ox, oy);
			break;
		case PETAL: 
			partColors[which] = getColorC(px, py);
			r_setColor(gr, partColors[which]);
			r_drawLine(gr, px, py, ox, oy);
			break;
		}		
		
		//r_setColor(gr, Color.red);
		//r_drawLine(gr, p2init, 0, p2init, 500);

	}	






	void growSpawn(int which) {
		int type;
		double age;
		
		type = (int)parts[which][TYPE];
		age = (double)simTicks - parts[which][BIRTHDAY];
		
		switch (type) {
			case SEED: 
				if (Math.random()<0.1) {
  					addPart(which, STEM1);
				}
				break;		
			case STEM1: 
				if (age<1000.0) {
					parts[which][LENGTH] = parts[which][STRENGTH] * age;
				} else {
					parts[which][LENGTH] = parts[which][STRENGTH] * 1000.0;
				}
				if (Math.random()<0.01) {
					addPart(which, STEM2);
				}
				break;
			case STEM2: 
				if (age<500.0) {
					parts[which][LENGTH] = parts[which][STRENGTH] * age;
				} else {
					parts[which][LENGTH] = parts[which][STRENGTH] * 500.0;
				}
				if (Math.random()<0.01) {
					if (Math.random()>0.5) {
	  					addPart(which, STEM2);
					} else { 
						addPart(which, STEM3);
					}
				}
				break;
			case STEM3: 
				if (age<2000.0) {
					parts[which][LENGTH] = parts[which][STRENGTH] * age;
				} else {
					parts[which][LENGTH] = parts[which][STRENGTH] * 2000.0;
				}
				if (Math.random()<0.1) {
					addPart(which, STEM1);
				}
				break;
			case PETAL: 
				if (age<300.0) {
					parts[which][LENGTH] = parts[which][STRENGTH] * age;
				} else {
					parts[which][LENGTH] = parts[which][STRENGTH] * 300.0;
				}
				break;		
		}
	}


	
	/* variable-set warning: add particles st parents 
		have indices < child indices, so that coords get
		generated from the center out. */
	void movePart(int which) {
		double ox, oy, px, py, dx, dy;
		int type, age, parent, rot;

		type = (int)parts[which][TYPE];
		
		age = simTicks - (int)(parts[which][BIRTHDAY]);
		parent = (int)parts[which][PARENT];

		rot = (int)(parts[which][DIR]) % 360;
		if (rot<0) rot += 360;

		/* everybody's direction changes a little */
		//dx = parts[which][DDDIR];
		//parts[which][DDIR] += ((Math.random()*dx) - (dx*0.5));
		
		//dx = parts[which][DDIR];
		//parts[which][DIR] += ((Math.random()*dx) - (dx*0.5));
		
		
		growSpawn(which); 
		
		// if you're a seed, grow where we putcha
		if (type==SEED) {
			parts[which][OX] = seedX;
			parts[which][OY] = seedY;
		} else {
			if (parent>=0) {
				parts[which][OX] = parts[parent][XPOS];
				parts[which][OY] = parts[parent][YPOS];
			}
			// else parent has died; leave OXY to mark the spot
			// else { }
		}
		
		ox = parts[which][OX] + parts[which][LENGTH] * sines[rot];
		oy = parts[which][OY] + parts[which][LENGTH] * cosines[rot];
		parts[which][XPOS] = ox;
		parts[which][YPOS] = oy;

	}




	void drawAll(Graphics g, int howMany){
		int i;
		// for the first 12 ticks, do image processing stuff.
		if ((mouseStop==0) && (slowdownCounter<1)) {
			if (simTicks<12) {
				loadImage(g, simTicks);
			} else {
				r_setColor(g, backgroundColor);
				r_fillRect(g, 0,0,size().width,size().height);
				for (i=0; i<howMany; ++i) {
					drawPart(g, i);
				}
			}
		} 
	}
}

